@@ -3,16 +3,18 @@ from pydantic import BaseModel, Field, root_validator, constr, conlist
 from typing import List, Union
 
 from honeybee_schema._base import IDdBaseModel
-from honeybee_schema.model import Face3D, Units
+from honeybee_schema.model import Room, Face3D, Mesh3D, Units
 from honeybee_schema.boundarycondition import Ground, Outdoors, Surface, \
     Adiabatic, OtherSideTemperature
 from honeybee_schema.altnumber import Autocalculate
 
-from .window_parameter import SingleWindow, SimpleWindowRatio, RepeatingWindowRatio, \
-    RectangularWindows, DetailedWindows
+from .window_parameter import SingleWindow, SimpleWindowArea, SimpleWindowRatio, \
+    RepeatingWindowRatio, RectangularWindows, DetailedWindows
 from .shading_parameter import ExtrudedBorder, Overhang, LouversByDistance, \
     LouversByCount
-from .skylight_parameter import GriddedSkylightRatio, DetailedSkylights
+from .skylight_parameter import GriddedSkylightArea, GriddedSkylightRatio, \
+    DetailedSkylights
+from .roof import RoofSpecification
 from .energy.properties import Room2DEnergyPropertiesAbridged, \
     StoryEnergyPropertiesAbridged, BuildingEnergyPropertiesAbridged, \
     ContextShadeEnergyPropertiesAbridged, ModelEnergyProperties
@@ -90,17 +92,19 @@ class Room2D(IDdBaseModel):
         'height of the room is at or below 0 (the assumed ground plane).'
     )
 
-    window_parameters: List[Union[None, SingleWindow, SimpleWindowRatio,
-                                  RepeatingWindowRatio, RectangularWindows,
-                                  DetailedWindows]] = Field(
+    window_parameters: List[Union[
+        None, SingleWindow, SimpleWindowArea, SimpleWindowRatio, RepeatingWindowRatio,
+        RectangularWindows, DetailedWindows
+    ]] = Field(
         default=None,
         description='A list of WindowParameter objects that dictate how the window '
         'geometries will be generated for each of the walls. If None, no windows '
         'will exist over the entire Room2D.'
     )
 
-    shading_parameters: List[Union[None, ExtrudedBorder, Overhang, LouversByDistance,
-                                   LouversByCount]] = Field(
+    shading_parameters: List[Union[
+        None, ExtrudedBorder, Overhang, LouversByDistance, LouversByCount
+    ]] = Field(
         default=None,
         description='A list of ShadingParameter objects that dictate how the shade '
         'geometries will be generated for each of the walls. If None, no shades '
@@ -116,7 +120,9 @@ class Room2D(IDdBaseModel):
         'without any windows.'
     )
 
-    skylight_parameters: Union[None, GriddedSkylightRatio, DetailedSkylights] = Field(
+    skylight_parameters: Union[
+        None, GriddedSkylightArea, GriddedSkylightRatio, DetailedSkylights
+    ] = Field(
         default=None,
         description='A SkylightParameter object describing how to generate skylights. '
         'If None, no skylights will exist on the Room2D.'
@@ -185,6 +191,7 @@ class Story(IDdBaseModel):
 
     floor_to_floor_height: Union[Autocalculate, float] = Field(
         Autocalculate(),
+        ge=0,
         description='A number for the distance from the floor plate of this story '
         'to the floor of the story above this one (if it exists). If Autocalculate, '
         'this value will be the maximum floor_to_ceiling_height of the input room_2ds.'
@@ -204,6 +211,15 @@ class Story(IDdBaseModel):
         'repeated over the height of the building.'
     )
 
+    roof: RoofSpecification = Field(
+        default=None,
+        description='An optional RoofSpecification object containing geometry '
+        'for generating sloped roofs over the Story. The RoofSpecification will only '
+        'affect the child Room2Ds that have a True is_top_exposed property '
+        'and it will only be utilized in translation to Honeybee when the Story '
+        'multiplier is 1. If None, all Room2D ceilings will be flat.'
+    )
+
     properties: StoryPropertiesAbridged = Field(
         ...,
         description='Extension properties for particular simulation engines '
@@ -229,14 +245,33 @@ class Building(IDdBaseModel):
     type: constr(regex='^Building$') = 'Building'
 
     unique_stories: List[Story] = Field(
-        ...,
+        default=None,
         description='An array of unique dragonfly Story objects that together form '
         'the entire building. Stories should generally be ordered from lowest '
-        'floor to highest floor. Note that, if a given Story is repeated several '
-        'times over the height of the building, the unique story included in this '
+        'floor to highest floor, though this is not required. Note that, if a '
+        'given Story is repeated several times over the height of the building '
+        'and this is represented by the multiplier, the unique story included in this '
         'list should be the first (lowest) story of the repeated floors.'
     )
 
+    room_3ds: List[Room] = Field(
+        default=None,
+        description='An optional array of 3D Honeybee Room objects for additional '
+        'Rooms that are a part of the Building but are not represented within '
+        'the unique_stories. This is useful when there are parts of the Building '
+        'geometry that cannot easily be represented with the extruded floor '
+        'plate and sloped roof assumptions that underlie Dragonfly Room2Ds '
+        'and RoofSpecification. Cases where this input is most useful include '
+        'sloped walls and certain types of domed roofs that become tedious to '
+        'implement with RoofSpecification. Matching the Honeybee Room.story '
+        'property to the Dragonfly Story.display_name of an object within the '
+        'unique_stories will effectively place the Honeybee Room on that Story '
+        'for the purposes of floor_area, exterior_wall_area, etc. However, note '
+        'that the Honeybee Room.multiplier property takes precedence over '
+        'whatever multiplier is assigned to the Dragonfly Story that the '
+        'Room.story may reference. (Default: None).'
+    )
+
     properties: BuildingPropertiesAbridged = Field(
         ...,
         description='Extension properties for particular simulation engines '
@@ -262,10 +297,10 @@ class ContextShade(IDdBaseModel):
 
     type: constr(regex='^ContextShade$') = 'ContextShade'
 
-    geometry: List[Face3D] = Field(
+    geometry: List[Union[Face3D, Mesh3D]] = Field(
         ...,
-        description='An array of planar Face3Ds that together represent the '
-        'context shade.'
+        description='An array of planar Face3Ds and or Mesh3Ds that together '
+        'represent the context shade.'
     )
 
     is_detached: bool = Field(
